#!/bin/bash
root=`dirname $0`
drv_path=/lib/modules/`uname -r`/kernel/drivers/scsi/
drv_name=mvloki
sys_bin=/usr/bin/
sys_lib=/usr/lib/
base=`basename $PWD`
package_dir=/opt/mvwam
pkg_drv=$package_dir/driver
pkg_bin=$package_dir/bin
pkg_lib=$package_dir/lib
script_dir=$package_dir/scripts
images_dir=$package_dir/images
log_file=$package_dir/install.log
pci_dev_id="11ab:8180"

wam_update_pkg()
{
	echo "Updating firmware package"
	#echo cdev_major is $cdev_major
	nr_hbas=`lspci -d $pci_dev_id | wc -l`
	nr_hbas=`expr $nr_hbas - 1`
	for i in `seq 0 $nr_hbas`; do

		board_id=`wamctl /dev/mvwam$i boardid`
		ret=$?
		if [ $ret -ne 0 ]; then
			bmajor=0
			bminor=0
		else
			bmajor=`wamctl /dev/mvwam$i boardid | cut -d '.' -f 1`
			bminor=`wamctl /dev/mvwam$i boardid | cut -d '.' -f 2`
		fi

		echo "Board id: $bmajor.$bminor"
		pkg_file=$images_dir/wam_hw_$bmajor.$bminor.wpkg
		pkg_file_x=$images_dir/wam_hw_$bmajor.x.wpkg
		if [ -e $pkg_file ]; then
			found=1
		elif [ -e $pkg_file_x ]; then
			found=1
			pkg_file=$pkg_file_x
		else
			pkg_file=""
			found=0
		fi

		if [ $found -eq 1 ]; then
			serial=`wamctl /dev/mvwam$i info | grep Device | \
					awk '{print \$3}'`
			echo $serial > /opt/mvwam/$serial:pkg_upgrade_board
			echo 'y' | \
			wamctl /dev/mvwam$i update pkg=$pkg_file
			if [ $? -eq 0 ]; then
				rm -f /opt/mvwam/$serial:pkg_upgrade_board
				ret=0
			else
				echo "Update failed for board $serial"
				echo "Update failed for board $serial" \
					>> $log_file
				ret=1
			fi	
		else
			echo "Unable to determine compatible package.  " \
				"Update incomplete."
		fi
	done

	return $ret
}

wam_update_bin()
{
	echo "Updating driver/tools binaries"
	echo "Updating driver/tools binaries" >> $log_file

	echo "cp $pkg_drv/$drv_name.ko $drv_path" >> $log_file
	echo "cp $pkg_bin/wamdiag $sys_bin" >> $log_file
	echo "cp $pkg_bin/wamctl $sys_bin" >> $log_file
	echo "cp $pkg_lib/libmvwam.so $sys_lib" >> $log_file
	echo "cp $package_dir/wam_startup /etc/rc.d/init.d/wam_startup" >> \
		$log_file

	# copy the driver
	cp $pkg_drv/$drv_name.ko $drv_path

	# Run depmod -a
	depmod -a

	# copy the binary to tool path
	cp $pkg_bin/wamdiag $sys_bin
	cp $pkg_bin/wamctl $sys_bin

	# copy the library to lib path
	cp $pkg_lib/libmvwam.so $sys_lib

	# add wam_startup to /etc/rcS.d/
	cp $package_dir/wam_startup /etc/rc.d/init.d/wam_startup

	for i in 1 2 3 5; do
		cd /etc/rc${i}.d
		rm -f S80wam_startup
		ln -s ../init.d/wam_startup S80wam_startup
		cd - >> /dev/null
	done
}

wam_load_drv()
{
	echo "Loading driver"
	echo "Loading driver" >> $log_file
	$pkg_drv/install.sh

}

echo "Upgrading package" > $log_file
echo "Upgrading package" > $package_dir/.pkg_upgrade

wam_update_bin

drv_loaded=`lsmod | grep -c mvloki`
if [ $drv_loaded -eq 1 ]; then
	# ask admin to reboot
	echo "Please reboot the system for the driver update to take effect"
	echo "Please reboot the system for the driver update to take effect" >> $log_file
else
	# load driver
	wam_load_drv
fi

rm -f $package_dir/.pkg_upgrade
echo "Marvell WAM Installation complete"
